---
title: sql_select
type: input
status: experimental
categories: ["Services"]
---

<!--
     THIS FILE IS AUTOGENERATED!

     To make changes please edit the contents of:
     lib/input/sql_select.go
-->

import Tabs from '@theme/Tabs';
import TabItem from '@theme/TabItem';

:::caution EXPERIMENTAL
This component is experimental and therefore subject to change or removal outside of major version releases.
:::
Executes a select query and creates a message for each row received.

Introduced in version 3.59.0.


<Tabs defaultValue="common" values={[
  { label: 'Common', value: 'common', },
  { label: 'Advanced', value: 'advanced', },
]}>

<TabItem value="common">

```yaml
# Common config fields, showing default values
input:
  label: ""
  sql_select:
    driver: ""
    dsn: ""
    table: ""
    columns: []
    where: ""
    args_mapping: ""
```

</TabItem>
<TabItem value="advanced">

```yaml
# All config fields, showing default values
input:
  label: ""
  sql_select:
    driver: ""
    dsn: ""
    table: ""
    columns: []
    where: ""
    args_mapping: ""
    prefix: ""
    suffix: ""
```

</TabItem>
</Tabs>

Once the rows from the query are exhausted this input shuts down, allowing the pipeline to gracefully terminate (or the next input in a [sequence](/docs/components/inputs/sequence) to execute).

## Examples

<Tabs defaultValue="Consume a Table (PostgreSQL)" values={[
{ label: 'Consume a Table (PostgreSQL)', value: 'Consume a Table (PostgreSQL)', },
]}>

<TabItem value="Consume a Table (PostgreSQL)">


Here we define a pipeline that will consume all rows from a table created within the last hour by comparing the unix timestamp stored in the row column "created_at":

```yaml
input:
  sql_select:
    driver: postgres
    dsn: postgres://foouser:foopass@localhost:5432/testdb?sslmode=disable
    table: footable
    columns: [ '*' ]
    where: created_at >= ?
    args_mapping: |
      root = [
        now().format_timestamp_unix() - 3600
      ]
```

</TabItem>
</Tabs>

## Fields

### `driver`

A database [driver](#drivers) to use.


Type: `string`  
Options: `mysql`, `postgres`, `clickhouse`, `mssql`.

### `dsn`

A Data Source Name to identify the target database.

#### Drivers

The following is a list of supported drivers and their respective DSN formats:

| Driver | Data Source Name Format |
|---|---|
| `clickhouse` | [`tcp://[netloc][:port][?param1=value1&...&paramN=valueN]`](https://github.com/ClickHouse/clickhouse-go#dsn)
| `mysql` | `[username[:password]@][protocol[(address)]]/dbname[?param1=value1&...&paramN=valueN]` |
| `postgres` | `postgres://[user[:password]@][netloc][:port][/dbname][?param1=value1&...]` |
| `mssql` | `sqlserver://[user[:password]@][netloc][:port][?database=dbname&param1=value1&...]` |

Please note that the `postgres` driver enforces SSL by default, you
can override this with the parameter `sslmode=disable` if required.


Type: `string`  

```yaml
# Examples

dsn: tcp://host1:9000?username=user&password=qwerty&database=clicks&read_timeout=10&write_timeout=20&alt_hosts=host2:9000,host3:9000

dsn: foouser:foopassword@tcp(localhost:3306)/foodb

dsn: postgres://foouser:foopass@localhost:5432/foodb?sslmode=disable
```

### `table`

The table to select from.


Type: `string`  

```yaml
# Examples

table: foo
```

### `columns`

A list of columns to select.


Type: `array`  

```yaml
# Examples

columns:
  - '*'

columns:
  - foo
  - bar
  - baz
```

### `where`

An optional where clause to add. Placeholder arguments are populated with the `args_mapping` field. Placeholders should always be question marks, and will automatically be converted to dollar syntax when the postgres driver is used.


Type: `string`  

```yaml
# Examples

where: type = ? and created_at > ?

where: user_id = ?
```

### `args_mapping`

An optional [Bloblang mapping](/docs/guides/bloblang/about) which should evaluate to an array of values matching in size to the number of placeholder arguments in the field `where`.


Type: `string`  

```yaml
# Examples

args_mapping: root = [ "article", now().format_timestamp("2006-01-02") ]
```

### `prefix`

An optional prefix to prepend to the select query (before SELECT).


Type: `string`  

### `suffix`

An optional suffix to append to the select query.


Type: `string`  


