---
title: sql_raw
type: processor
status: stable
categories: ["Integration"]
---

<!--
     THIS FILE IS AUTOGENERATED!

     To make changes please edit the contents of:
     lib/processor/sql_raw.go
-->

import Tabs from '@theme/Tabs';
import TabItem from '@theme/TabItem';

Runs an arbitrary SQL query against a database and (optionally) returns the result as an array of objects, one for each row returned.

Introduced in version 3.65.0.


<Tabs defaultValue="common" values={[
  { label: 'Common', value: 'common', },
  { label: 'Advanced', value: 'advanced', },
]}>

<TabItem value="common">

```yaml
# Common config fields, showing default values
label: ""
sql_raw:
  driver: ""
  dsn: ""
  query: ""
  args_mapping: ""
  exec_only: false
```

</TabItem>
<TabItem value="advanced">

```yaml
# All config fields, showing default values
label: ""
sql_raw:
  driver: ""
  dsn: ""
  query: ""
  unsafe_dynamic_query: false
  args_mapping: ""
  exec_only: false
```

</TabItem>
</Tabs>

If the query fails to execute then the message will remain unchanged and the error can be caught using error handling methods outlined [here](/docs/configuration/error_handling).

## Examples

<Tabs defaultValue="Table Insert (MySQL)" values={[
{ label: 'Table Insert (MySQL)', value: 'Table Insert (MySQL)', },
{ label: 'Table Query (PostgreSQL)', value: 'Table Query (PostgreSQL)', },
]}>

<TabItem value="Table Insert (MySQL)">

The following example inserts rows into the table footable with the columns foo, bar and baz populated with values extracted from messages.

```yaml
pipeline:
  processors:
    - sql_raw:
        driver: mysql
        dsn: foouser:foopassword@tcp(localhost:3306)/foodb
        query: "INSERT INTO footable (foo, bar, baz) VALUES (?, ?, ?);"
        args_mapping: '[ document.foo, document.bar, meta("kafka_topic") ]'
        exec_only: true
```

</TabItem>
<TabItem value="Table Query (PostgreSQL)">

Here we query a database for columns of footable that share a `user_id` with the message field `user.id`. A [`branch` processor](/docs/components/processors/branch) is used in order to insert the resulting array into the original message at the path `foo_rows`.

```yaml
pipeline:
  processors:
    - branch:
        processors:
          - sql_raw:
              driver: postgres
              dsn: postgres://foouser:foopass@localhost:5432/testdb?sslmode=disable
              query: "SELECT * FROM footable WHERE user_id = $1;"
              args_mapping: '[ this.user.id ]'
        result_map: 'root.foo_rows = this'
```

</TabItem>
</Tabs>

## Fields

### `driver`

A database [driver](#drivers) to use.


Type: `string`  
Options: `mysql`, `postgres`, `clickhouse`, `mssql`.

### `dsn`

A Data Source Name to identify the target database.

#### Drivers

The following is a list of supported drivers and their respective DSN formats:

| Driver | Data Source Name Format |
|---|---|
| `clickhouse` | [`tcp://[netloc][:port][?param1=value1&...&paramN=valueN]`](https://github.com/ClickHouse/clickhouse-go#dsn)
| `mysql` | `[username[:password]@][protocol[(address)]]/dbname[?param1=value1&...&paramN=valueN]` |
| `postgres` | `postgres://[user[:password]@][netloc][:port][/dbname][?param1=value1&...]` |
| `mssql` | `sqlserver://[user[:password]@][netloc][:port][?database=dbname&param1=value1&...]` |

Please note that the `postgres` driver enforces SSL by default, you
can override this with the parameter `sslmode=disable` if required.


Type: `string`  

```yaml
# Examples

dsn: tcp://host1:9000?username=user&password=qwerty&database=clicks&read_timeout=10&write_timeout=20&alt_hosts=host2:9000,host3:9000

dsn: foouser:foopassword@tcp(localhost:3306)/foodb

dsn: postgres://foouser:foopass@localhost:5432/foodb?sslmode=disable
```

### `query`

The query to execute.


Type: `string`  

```yaml
# Examples

query: INSERT INTO footable (foo, bar, baz) VALUES (?, ?, ?);
```

### `unsafe_dynamic_query`

Whether to enable [interpolation functions](/docs/configuration/interpolation/#bloblang-queries) in the query. Great care should be made to ensure your queries are defended against injection attacks.


Type: `bool`  
Default: `false`  

### `args_mapping`

An optional [Bloblang mapping](/docs/guides/bloblang/about) which should evaluate to an array of values matching in size to the number of placeholder arguments in the field `query`.


Type: `string`  

```yaml
# Examples

args_mapping: root = [ this.cat.meow, this.doc.woofs[0] ]

args_mapping: root = [ meta("user.id") ]
```

### `exec_only`

Whether the query result should be discarded. When set to `true` the message contents will remain unchanged, which is useful in cases where you are executing inserts, updates, etc.


Type: `bool`  
Default: `false`  


